/**
 * Copyright (c) 2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.clientserverapi.topology.exception.AddContainerException;
import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.jmx.api.api.TopologyServiceClient;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;

/**
 * Client API of the JMX topology service.
 * 
 * @author Christophe Deneux - Linagora
 */
public final class TopologyServiceClientImpl extends AbstractServiceClient implements
        TopologyServiceClient {

    /**
     * Action : addContainerInformation
     */
    private static final String ADD_CONTAINER_CONFIGURATION = "addContainerInformation";

    /**
     * Creates a new instance of {@link InstallationServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     * @throws InstallationServiceDoesNotExistException
     * @throws InstallationServiceErrorException
     */
    public TopologyServiceClientImpl(String petalsDomain,
            MBeanServerConnection mBeanServerConnection)
            throws TopologyServiceDoesNotExistException, TopologyServiceErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, TopologyServiceClient.TOPOLOGY_SERVICE_MBEAN_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            ObjectName objName = new ObjectName(petalsDomain, attributes);
            Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new TopologyServiceDoesNotExistException();
            }
        } catch (MalformedObjectNameException e) {
            throw new TopologyServiceErrorException(e);
        } catch (IOException e) {
            throw new TopologyServiceErrorException(e);
        }
    }

    /**
     * {@inheritDoc}
     * 
     * @throws TopologyServiceErrorException
     *             An unexpected error occurs performing the JMX operation
     */
    @Override
    public void addContainerConfiguration(final ContainerConfiguration addedContainer,
            final String subdomainName) throws AddContainerException, PetalsException {

        final Object[] objects = new Object[] { addedContainer, subdomainName };
        final String[] types = new String[] { ContainerConfiguration.class.getName(),
                String.class.getName() };

        this.performAddContainerConfiguration(objects, types);

    }

    /**
     * {@inheritDoc}
     * 
     * @throws TopologyServiceErrorException
     *             An unexpected error occurs performing the JMX operation
     */
    @Override
    public void addContainerConfiguration(ContainerConfiguration addedContainer,
            String subdomainName, boolean validateTopology) throws AddContainerException,
            PetalsException {

        final Object[] objects = new Object[] { addedContainer, subdomainName, validateTopology };
        final String[] types = new String[] { ContainerConfiguration.class.getName(),
                String.class.getName(), boolean.class.getName() };

        this.performAddContainerConfiguration(objects, types);

    }

    private void performAddContainerConfiguration(final Object[] objects, final String[] types)
            throws AddContainerException, TopologyServiceErrorException {

        try {
            this.performAction(ADD_CONTAINER_CONFIGURATION, objects, types);
        } catch (final PerformActionErrorException e) {
            if (e.getCause() instanceof AddContainerException) {
                throw (AddContainerException) e.getCause();
            } else {
                throw new TopologyServiceErrorException(e);
            }
        }
    }
}
